Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 279138)
+++ channels/chan_sip.c	(working copy)
@@ -2338,21 +2338,39 @@
 static int send_provisional_keepalive_full(struct sip_pvt *pvt, int with_sdp)
 {
 	const char *msg = NULL;
+	int res = 0;
 
+	ast_mutex_lock(&pvt->lock);
+	while (pvt->owner && ast_channel_trylock(pvt->owner)) {
+		DEADLOCK_AVOIDANCE(&pvt->lock);
+	}
+
+	if (pvt->provisional_keepalive_sched_id == -1) {
+		goto provisional_keepalive_cleanup;
+	}
+
+	if (pvt->invitestate >= INV_COMPLETED) {
+		pvt->provisional_keepalive_sched_id = -1; /* if we don't re-schedule, make sure to remove the sched id */
+		goto provisional_keepalive_cleanup;
+	}
+
 	if (!pvt->last_provisional || !strncasecmp(pvt->last_provisional, "100", 3)) {
 		msg = "183 Session Progress";
 	}
 
-	if (pvt->invitestate < INV_COMPLETED) {
-		if (with_sdp) {
-			transmit_response_with_sdp(pvt, S_OR(msg, pvt->last_provisional), &pvt->initreq, XMIT_UNRELIABLE);
-		} else {
-			transmit_response(pvt, S_OR(msg, pvt->last_provisional), &pvt->initreq);
-		}
-		return PROVIS_KEEPALIVE_TIMEOUT;
+	if (with_sdp) {
+		transmit_response_with_sdp(pvt, S_OR(msg, pvt->last_provisional), &pvt->initreq, XMIT_UNRELIABLE);
+	} else {
+		transmit_response(pvt, S_OR(msg, pvt->last_provisional), &pvt->initreq);
 	}
 
-	return 0;
+	res = PROVIS_KEEPALIVE_TIMEOUT;
+provisional_keepalive_cleanup:
+	if (pvt->owner) {
+		ast_channel_unlock(pvt->owner);
+	}
+	ast_mutex_unlock(&pvt->lock);
+	return res;
 }
 
 static int send_provisional_keepalive(const void *data) {
